#define MAXLINE 1024
#define MAX_WORD_LENGTH 50
#define MAXDBS 60
   /* maximum number of databases for which to report "recall"
      values (i.e., recall values considering we include up to
      MAXDBS in the GlOSS rank) */

#define GLOSS_DIR "/u/gravano/ggloss/experiments"
#define QUERY_DIR GLOSS_DIR"/queries"
#define NEWS_DIR "/usr/spool/news"
#define DBS_DIR GLOSS_DIR"/dbs"
#define NTC_NNN_DIR DBS_DIR"/ntc.nnn"
#define NNN_NNN_DIR DBS_DIR"/nnn.nnn"
#define GNUPLOT_DIR GLOSS_DIR"/gnuplot_data"
#define LOCAL_DIR "/usr/tmp"

#define DB_LIST DBS_DIR"/dblist"  
   /* contains the list of the directories that define each of the 
      databases that Smart will index */

#define PROC_DB_LIST DB_LIST".proc"
   /* list of directory-dir_id pairs */

#define SIFT_QUERIES "sift_queries"
#define RES_SUMMARY NTC_NNN_DIR"/res_summary"
#define REAL_RANK   NTC_NNN_DIR"/real_rank"
#define GLOSS_RANK  NTC_NNN_DIR"/gloss_rank"

#define TEST_NTC_NNN GLOSS_DIR"/make_smart/test_news.ntc.nnn"
#define TEST_NNN_NNN GLOSS_DIR"/make_smart/test_news.nnn.nnn"

#define SMPRINT GLOSS_DIR"/smart.11.0/src/bin/smprint"
#define RANKER  GLOSS_DIR"/src/sort_rank" 

#define REAL_RANK_W 1 
        /* Algorithm for ranking the databases according to their
           goodness for a given query: take as this "goodness"
           the sum of all the weights of the documents in the
           answer to the query that are higher than or equal 
           to a given threshold*/

#define REAL_RANK_D 2
        /* Algorithm for ranking the databases according to their
           goodness for a given query: take as this "goodness"
           the number of documents in the answer to the query
           with weight higher than or equal to a given threshold */

#define GLOSS_RANK_MAX_W 3
#define GLOSS_RANK_MAX_D 4
        /* Algorithm for ranking the databases according to their
           estimated goodness for a given query: GlOSS takes as this
           "goodness" the estimated number of documents in the given database
           that will have weight higher than or equal to a given 
           threshold (GLOSS_RANK_MAX_D), or the sum of the similarity
           of these documents (GLOSS_RANK_MAX_W).
           To estimate these numbers, GlOSS assumes that 
           the weight of each word in a database is uniformly distributed
           among the documents that contain that word, and that 
           the query words occur in documents with maximum correlation
           (Max... ) */

#define GLOSS_RANK_SUM_W 5
#define GLOSS_RANK_SUM_D 6
        /* Idem GLOSS_RANK_MAX_*, but assuming that the query words
           occur in documents with negative correlation (Sum...) */ 

#define PRUNED_RANKS 'P'
#define NON_PRUNED_RANKS 'N'

/* Structure to keep some of the parameters of the different experiments */
typedef struct type_exp{
        int real_rank_alg;  /* Algorithm for ranking the databases
                               according to their goodness for a given
                               query */
        int gloss_rank_alg; /* Algorithm for GlOSS to estimate the
                               goodness of the databases for a given query */
        float threshold_real; /* Parameter to some of the options of the
                                 ideal ranking algorithms */ 
        float threshold_gloss;/* Parameter to some of the options of the
                                 gloss ranking algorithms */ 
        char output_file[FILENAME_MAX];
                            /* File where the results of the experiment 
                               will be saved. */  
        char prune;         /* PRUNED_RANKS if the GlOSS ranks are
                                       truncated with the first database
                                       with a zero estimated goodness.
                               NON_PRUNED_RANKS if not */
} t_exp; 

/* Structure to keep the results of the (sub)queries */ 
typedef struct type_qres{
        long doc_counts; 
        float weight_sums;
} t_qres;

