/************************************************************************/
#include <stdio.h>
#include <strings.h>
#include "filenames.h"

#define N_PARAM 3

/************************************************************************/
void Panic(char *);

/************************************************************************/
void main(int argc, char **argv){
char f_results[FILENAME_MAX], f_gpdata[FILENAME_MAX],
     f_gpdata_2[FILENAME_MAX], f_gpdata_3[FILENAME_MAX];
FILE *results, *gpdata, *gpdata2, *gpdata3=(FILE *)NULL;
int param_1;
float thr_real, thr_gloss;
char real_alg='\0';
int num_dbs;
float precision, recall;

char tmp[MAXLINE], buffer[MAXLINE], gloss_alg[MAXLINE];

if(argc!=2){
  fprintf(stderr, "Usage: %s results_file_name\n", argv[0]);
  exit(0);
}

sprintf(f_results, "%s/%s", NTC_NNN_DIR, argv[1]);
if((results=fopen(f_results, "r"))==(FILE *)NULL)
  Panic("Couldn't open results file");

/* parse the results file, and generate the gnuplot-data file:
   the gnuplot filename will be determined by the parameters
   in the results file */
 
while(fgets(buffer, MAXLINE, results)!=NULL){
 if(strstr(buffer, "Real_rank_algorithm:")!=NULL){

 if(sscanf(buffer, "%s %d", tmp, &param_1)!=2)
   Panic("Bad real_rank line");

 switch(param_1){
  case REAL_RANK_W: real_alg='w';
                    break;
  case REAL_RANK_D: real_alg='d';
                    break;
  default: Panic("Unknown real_rank algorithm");
 } 

 if((fgets(buffer, MAXLINE, results)==NULL)||
    (strstr(buffer, "GlOSS_rank_algorithm:")==NULL) ||
    (sscanf(buffer, "%s %d", tmp, &param_1)!=2))
   Panic("Bad gloss_rank line"); 

 switch(param_1){
  case GLOSS_RANK_MAX_W: strcpy(gloss_alg, "maxw");
                         break; 

  case GLOSS_RANK_MAX_D: strcpy(gloss_alg, "maxd");
                         break; 
    
  case GLOSS_RANK_SUM_W: strcpy(gloss_alg, "sumw");
                         break; 

  case GLOSS_RANK_SUM_D: strcpy(gloss_alg, "sumd");
                         break; 

  default: Panic("Unknown gloss_rank algorithm");
 } 
     
 if((fgets(buffer, MAXLINE, results)==NULL)||
    (strstr(buffer, "Threshold_Real:")==NULL) ||
    (sscanf(buffer, "%s %f", tmp, &thr_real)!=2))
   Panic("Bad Threshold_Real line"); 

 if((fgets(buffer, MAXLINE, results)==NULL)||
    (strstr(buffer, "Threshold_GlOSS:")==NULL) ||
    (sscanf(buffer, "%s %f", tmp, &thr_gloss)!=2))
   Panic("Bad Threshold_GlOSS line"); 

 /* generate gnuplot-data filenames */ 
 sprintf(f_gpdata, "%s/gp_res_%s%d_%c%d", GNUPLOT_DIR, gloss_alg,
         (int)(thr_gloss*100), real_alg, (int)(thr_real*100));


 if(thr_gloss==thr_real)
   sprintf(f_gpdata_2, "%s/gp_thr_%s_%c_n%d", GNUPLOT_DIR, gloss_alg,
           real_alg, N_PARAM);
 else
   sprintf(f_gpdata_2, "%s/gp_thr_%s0_%c_n%d", GNUPLOT_DIR, gloss_alg,
           real_alg, N_PARAM);

 if((thr_gloss==thr_real)&&(thr_gloss==0))
   sprintf(f_gpdata_3, "%s/gp_thr_%s0_%c_n%d", GNUPLOT_DIR, gloss_alg,
           real_alg, N_PARAM);

 if((gpdata=fopen(f_gpdata, "w"))==(FILE *)NULL)
   Panic("Can't create gnuplot-data file");

 if((gpdata2=fopen(f_gpdata_2, "a"))==(FILE *)NULL)
   Panic("Can't append to gnuplot-data file 2");

 if((thr_gloss==thr_real)&&(thr_gloss==0))
   if((gpdata3=fopen(f_gpdata_3, "a"))==(FILE *)NULL)
     Panic("Can't append to gnuplot-data file 3");

 while((fgets(buffer, MAXLINE, results)!=NULL)&&
       (strstr(buffer, "Num_DBs")==NULL));

 /* Now we got to the real data... */
 while((fgets(buffer, MAXLINE, results)!=NULL)&&
       (strstr(buffer, "----")==NULL)){
   fprintf(gpdata, "%s", buffer);
   sscanf(buffer, "%d %f %f", &num_dbs, &recall, &precision);
   if(num_dbs==N_PARAM){
     fprintf(gpdata2, "%d %f %f %f\n", num_dbs, thr_real, recall, precision);
     if((thr_gloss==thr_real)&&(thr_gloss==0))
       fprintf(gpdata3, "%d %f %f %f\n", num_dbs, thr_real, recall, precision);
   }
 }

 fclose(gpdata); 
 fclose(gpdata2);
 if((thr_gloss==thr_real)&&(thr_gloss==0))
   fclose(gpdata3);

 printf("Generated: \t%s\n\t\t%s\n\n", f_gpdata, f_gpdata_2);
 if((thr_gloss==thr_real)&&(thr_gloss==0))
   printf("Also:\t\t%s\n\n", f_gpdata_3);
 }
}
fclose(results);
}

/************************************************************************/
void Panic(char *msg){
fprintf(stderr, "Problem: %s\n", msg);
exit(0);
} 

/************************************************************************/
