#define MAXLINE 1024
#define MAX_WORD_LENGTH 50
#define MAXCLUSTERS 60
   /* maximum number of database clusters for which to report statistics 
      (i.e., statistics considering we include up to MAXCLUSTERS in 
       the HGlOSS rank) */

#define GLOSS_DIR "/u/gravano/ggloss/experiments"
#define QUERY_DIR GLOSS_DIR"/queries"
#define DBS_DIR GLOSS_DIR"/dbs"
#define NTC_NNN_DIR DBS_DIR"/ntc.nnn"

/* list of database clusters: each database cluster corresponds
   to the databases indexed by a GGlOSS server */
#define PROC_CLUSTER_LIST DBS_DIR"/cluster_list.proc"

#define SIFT_QUERIES "sift_queries"
#define H_RES_SUMMARY  NTC_NNN_DIR"/h_res_summary"
#define H_REAL_RANK    NTC_NNN_DIR"/h_real_rank"
#define HGLOSS_RANK  NTC_NNN_DIR"/hgloss_rank"
#define CLUSTER_DATA NTC_NNN_DIR"/cluster_data"

#define RANKER  GLOSS_DIR"/src/sort_rank" 

#define H_REAL_RANK_W 1 
        /* Algorithm for ranking the clusters according to their
           goodness for a given query: take as this "goodness"
           the sum of all the estimated goodnesses in the
           answer to the query that are higher than or equal 
           to a given threshold */

#define H_REAL_RANK_D 2
        /* Algorithm for ranking the clusters according to their
           goodness for a given query: take as this "goodness"
           the number of databases in the answer to the query
           with estimated goodness higher than or equal 
           to a given threshold */

#define GLOSS_RANK_MAX_W 3
#define GLOSS_RANK_MAX_D 4
        /* Algorithm for ranking the databases according to their
           estimated goodness for a given query: GlOSS takes as this
           "goodness" the estimated number of documents in the given database
           that will have weight higher than or equal to a given 
           threshold (GLOSS_RANK_MAX_D), or the sum of the similarity
           of these documents (GLOSS_RANK_MAX_W).
           To estimate these numbers, GlOSS assumes that 
           the weight of each word in a database is uniformly distributed
           among the documents that contain that word, and that 
           the query words occur in documents with maximum correlation
           (Max... ) */

#define GLOSS_RANK_SUM_W 5
#define GLOSS_RANK_SUM_D 6
        /* Idem GLOSS_RANK_MAX_*, but assuming that the query words
           occur in documents with negative correlation (Sum...) */ 

#define H_GLOSS_RANK_MAX_W 7
#define H_GLOSS_RANK_MAX_D 8
        /* Algorithm for ranking the database clusters according to their
           estimated goodness for a given query: HGlOSS takes as this
           "goodness" the estimated number of databases in the given cluster 
           that will have estimated goodness higher than or equal to a given 
           threshold (H_GLOSS_RANK_MAX_D), or the sum of the estimated goodness 
           of these documents (H_GLOSS_RANK_MAX_W). */

#define PRUNED_RANKS 'P'
#define NON_PRUNED_RANKS 'N'

/* Structure to keep some of the parameters of the different experiments */
typedef struct type_exp{
        int h_real_rank_alg;  /* Algorithm for ranking the database clusters 
                               according to their goodness for a given
                               query */
        int gloss_rank_alg;   /* Algorithm for GlOSS to estimate the
                               goodness of the databases for a given query */
        int hgloss_rank_alg; /* Algorithm for HGlOSS to estimate the
                               goodness of the database clusters for a given
                               query */
        float threshold_h_real; /* Parameter to some of the options of the
                                 ideal ranking algorithms */ 
        float threshold_gloss;  /* Parameter to some of the options of the
                                 gloss ranking algorithms */ 
        float threshold_hgloss; /* Parameter to some of the options of the
                                 hgloss ranking algorithms */
        char output_file[FILENAME_MAX];
                            /* File where the results of the experiment 
                               will be saved. */  
        char prune;         /* PRUNED_RANKS if the GlOSS ranks are
                                       truncated with the first database
                                       with a zero estimated goodness.
                               NON_PRUNED_RANKS if not */
} t_exp; 

/* Structure to keep the results of the (sub)queries */ 
typedef struct type_qres{
        long doc_counts; 
        float weight_sums;
} t_qres;

