/***********************************************************************/
#include <strings.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "h_filenames.h"
#include "hgloss.h"

/***********************************************************************/
void Panic(char *);
int GetNextWord(char **, char *);
void GenerateQueryWeightVector(char *, int, int *);
void OpenQueryFiles(char *, char *, char *, FILE **, FILE **, FILE **);
void CloseQueryFiles(FILE *, FILE *, FILE *, char *);
void ProcessQueryFiles(char *, long, FILE *, FILE *, FILE *, t_exp *);
int ProcessQueryData(int *, char *, long, FILE *, FILE *, FILE *, t_exp *);
int ReadQuery(FILE *, int *, int **);
void ReadAndCollapseResults(FILE *, int, int, t_qres *, t_exp *);
void ReadAndCollapseResultsForOne(FILE *, int, long *, float *, int, t_exp *);
float GlOSSGoodness(int, t_qres *, long, t_exp *, int *);
int query_results_compare(void *, void *);
void OpenRankFiles(char *, char *, FILE **, FILE **);
void ComputeStats(int, FILE *, FILE *, t_exp *);
void CloseRankFiles(FILE *, FILE *);
void CreateVectors(int, float **, float **, float **, float *, float *);
void DestroyVectors(float *, float *, float *);
int ReadInOneQuery(int, FILE *, FILE *, float *, float *, float *);
void AnalyzeData(int, float *, float *, float *, float *, float *, t_exp *);
void ReportResults(int, int, float *, float *, t_exp *);
void CollapseClusterData(char *, int, char *, char *, t_exp *);
void ProcessQueryResults(char *, long, char *, char *, t_exp *);

/***********************************************************************/

char gbuffer[MAXLINE]; /* global variable, keeping the last
     line that we read from a query result file, which we shouldn't
     have read. Ungetc only guarantees ungetting one char.
     Horrible patch. Temporary... */

long zero_real_goodness=0, zero_hgloss_goodness=0, total_goodness=0;

/***********************************************************************/
void main(int argc, char *argv[]){
int num_clusters;
t_exp experiment; 

if(argc!=8){
  fprintf(stderr, "Usage %s: \n\tH_Real_Rank_Algorithm [1..2]\n \
\tGlOSS_Rank_Algorithm [3..6] \n\
\tHGlOSS_Rank_Algorithm [7..8] \n \tThreshold_H_Real [0..\?]\n\
\tThreshold_GlOSS [0..1]\n\
\tThreshold_HGlOSS [0..0]\n \tOutput_File (without path)\n", argv[0]);  
  exit(0);
} 

/* initialize global gbuffer */
gbuffer[0]='\0';

/* Set some experiment parameters */
experiment.h_real_rank_alg   = atoi(argv[1]);
experiment.gloss_rank_alg    = atoi(argv[2]);
experiment.hgloss_rank_alg   = atoi(argv[3]);
sscanf(argv[4], "%f", &(experiment.threshold_h_real)); 
sscanf(argv[5], "%f", &(experiment.threshold_gloss)); 
sscanf(argv[6], "%f", &(experiment.threshold_hgloss)); 
sprintf(experiment.output_file, "%s/%s", NTC_NNN_DIR, argv[7]);
experiment.prune=PRUNED_RANKS;

/* Run experiments */

GenerateRanks(PROC_CLUSTER_LIST, SIFT_QUERIES, NTC_NNN_DIR, 
              &num_clusters, &experiment);

CompareRanks(num_clusters, &experiment);

}

/***********************************************************************/
int GetNextWord(char **buffer, char *word){
  int i=0;

  /* skip white space, and the line break if at the end of the query...*/
  while(((*buffer)[i]==' ')||((*buffer)[i]=='\n'))
    i++;

  if((*buffer)[i]=='\0')
    return EOF;

  /* There is a word to read */
  (*buffer)=(*buffer)+i;
  i=0;

  while(((*buffer)[i]!='\0') && ((*buffer)[i]!=' ') &&
        ((*buffer)[i]!='\n')){
    word[i]=(*buffer)[i];
    i++;
  }

  word[i]='\0';

  /* advance buffer pointer after the current word ... */
  (*buffer)=(*buffer)+i;

  return 0;
}

/***********************************************************************/
void GenerateQueryWeightVector(char *query, int num_words, int *w_vector){
/* Returns in array w_vector the query weight vector associated with
   query. Query has num_words words. The vector associated with
   query "a b c a a" will be "3 1 1 0 0," for example. */

char *terms;
int i,j;

/* load the num_words query terms into array terms
   and initialize w_vector */
terms=malloc(sizeof(char)*MAXLINE*num_words);

for(i=0; i<num_words; i++){
  w_vector[i]=-1;
  if(GetNextWord(&query, (terms+i*MAXLINE))==EOF)
    Panic("Can't parse query for generating query weights");
}

/* generate weights */
for(i=0; i<num_words; i++){
  if(w_vector[i]==-1){
    /* word not found earlier in the query: process */
    w_vector[i]=1;

    for(j=i+1; j<num_words; j++)
      /* check if word appears later in query */
      if(strcmp(terms+i*MAXLINE, terms+j*MAXLINE)==0){
        /* found word later */
        w_vector[i]++;
        w_vector[j]=0;
      }
  }
}
free(terms);
} 

/***********************************************************************/
void Panic(char *msg){
fprintf(stderr, "Problem: %s\n", msg);
exit(0);
}

/***********************************************************************/
void ProcessQueryResults(char *dbid, long num_docs, 
      char *f_queries, char *res_dir, t_exp *experiment){

/* f_queries.mst is a file containing the original set of queries.
   res_dir is a directory containing the result of evaluating the
   queries in f_queries.mst on database dbid, which has
   num_docs in total.
   File CLUSTER_DATA will contain the "summary" information
   (see below) for each query and database dbid of the current cluster.

   This function processes the query results and comes up with 
   GlOSS's attempt at capturing the goodness of dbid for the queries.
   (The GlOSS algorithm is specified in experiment.) */

FILE *queries, *results, *proc_results;
char rf_queries[FILENAME_MAX], f_results[FILENAME_MAX];

/* Generate the file names...*/
sprintf(rf_queries, "%s/%s.mst", QUERY_DIR, f_queries);
sprintf(f_results, "%s/%s.txt", res_dir, dbid);

OpenQueryFiles(rf_queries, f_results, CLUSTER_DATA,
               &queries, &results, &proc_results);
ProcessQueryFiles(dbid, num_docs, queries, results, proc_results, experiment);
CloseQueryFiles(queries, results, proc_results, f_results);
}

/***********************************************************************/
void OpenQueryFiles(char *rf_queries, char *f_results, char *f_proc_results,
                    FILE **queries, FILE **results, FILE **proc_results){

char buffer[MAXLINE];

if((*queries=fopen(rf_queries, "r"))==(FILE *)NULL)
  Panic("Couldn't open queries file");

/* uncompress query-results file */
sprintf(buffer, "gunzip %s\n", f_results);
system(buffer); 

if((*results=fopen(f_results, "r"))==(FILE *)NULL)
  Panic("Couldn't open results file");

if((*proc_results=fopen(f_proc_results, "a"))==(FILE *)NULL)
  Panic("Couldn't create proc_results file");
}

/***********************************************************************/
void CloseQueryFiles(FILE *queries, FILE *results, FILE *proc_results,
                     char *f_results){

char buffer[MAXLINE];

fclose(queries);
fclose(results);
fclose(proc_results);

/* Compress the query-results file */
sprintf(buffer, "gzip %s\n", f_results);
system(buffer);
}

/***********************************************************************/
void ProcessQueryFiles(char *dbid, long num_docs, 
                       FILE *queries, FILE *results, FILE *proc_results,
                       t_exp *experiment){
/* For each query in queries, take the results for it and its individual
   one-keyword subqueries from results, and combine these results
   the "GlOSS-predicted" goodness number, using the algorithm 
   specified in experiment. */

int qid=1;  /* keeps track of the current query number */
            /* Smart numbers queries starting from 1 */

while(ProcessQueryData(&qid, dbid, num_docs, queries, results, proc_results, 
                       experiment)!=EOF);
     
}

/***********************************************************************/
int ProcessQueryData(int *qid, char *dbid, long num_docs, 
                     FILE *queries, FILE *results, FILE *proc_results,
                     t_exp *experiment){
/* Takes the next query (query number qid) from file queries and its
   associated one-keyword queries, and the corresponding results from
   file results, and generates the "GlOSS-predicted" goodness number, 
   using the algorithm described in experiment.

   Returns EOF if there are no more queries to process. */

int num_terms;
int result;
t_qres *query_results, *query_results_tmp;

float gloss_good;

int *query_weights, i;

if((result=ReadQuery(queries, &num_terms, &query_weights))==EOF)
  return EOF;

/* Creates the query_results vector */
if((query_results=(t_qres *)malloc(sizeof(t_qres)*(num_terms+1)))==NULL)
   Panic("Malloc failed");

ReadAndCollapseResults(results, *qid, num_terms, query_results, experiment);

/* save copy of array query_results_tmp: it will change in GlOSSGoodness */
query_results_tmp=(t_qres *)malloc(sizeof(t_qres)*(num_terms+1));
for(i=0; i<=num_terms; i++){
  query_results_tmp[i].doc_counts= query_results[i].doc_counts;
  query_results_tmp[i].weight_sums= query_results[i].weight_sums; 
}

gloss_good=GlOSSGoodness(num_terms, query_results, num_docs, experiment,
                         query_weights);

/* Output format: if query qid is "t1 ... tn":

   <qid>  n <dbid> <estimated "goodness">
   <qid+1> <dbid> <accum. weight of t1 in dbid> <doc. count for t1 in dbid>
   ...
   <qid+n> <dbid> <accum. weight of tn in dbid> <doc. count for tn in dbid>
 */

fprintf(proc_results, "%d\t%d %s %f\n", *qid, num_terms, dbid, gloss_good);

for(i=1; i<=num_terms; i++){
   fprintf(proc_results, "%d\t%s %f %ld\n", (*qid)+i, dbid,
           query_results_tmp[i].weight_sums, query_results_tmp[i].doc_counts);
}

free(query_results);
free(query_results_tmp);
free(query_weights);

(*qid)=(*qid)+num_terms+1;
return *qid;
}

/***********************************************************************/
int ReadQuery(FILE *queries, int *num_terms, int **query_weights){
/* Read next query and its subqueries from file queries.
   Returns in vector query_weights the weights associated with each 
   of the num_terms terms of the current query.
   Returns EOF if there are no more queries to process. */

char orig_query[MAXLINE];
char keyword[MAXLINE];
char buffer[MAXLINE];
int i;

/* First read number of keywords in the query */
if(fgets(buffer, MAXLINE, queries)==NULL)
  return EOF;

if(sscanf(buffer, "%d", num_terms)!=1)
  Panic("Query file in wrong format");

/* Now read the entire query */
if(fgets(orig_query, MAXLINE, queries)==NULL)
  Panic("Couldn't read original query from queries file");

/* And the num_terms individual keywords that compose the query */
for(i=0; i<(*num_terms); i++)
   if(fgets(keyword, MAXLINE, queries)==NULL)
     Panic("Couldn't read all keywords from queries file");

(*query_weights) = malloc(sizeof(int)*(*num_terms));
GenerateQueryWeightVector(orig_query, (*num_terms), (*query_weights));

return 0;
}

/***********************************************************************/
void ReadAndCollapseResults(FILE *results, int qid, int num_terms, 
                            t_qres *query_results, t_exp *experiment){

/* Reads the query results associated with query number qid and all of
   its num_terms subqueries from file results. 
   Then combines these results into vector query_results
   using the algorithms described in experiment. */

int i;
long count;
float sum;

for(i=0; i<=num_terms; i++){
   ReadAndCollapseResultsForOne(results, qid+i, &count, &sum, i, experiment); 
   query_results[i].doc_counts=count;
   query_results[i].weight_sums=sum;
}

return;
}

/***********************************************************************/
void ReadAndCollapseResultsForOne(FILE *results, int qid, 
            long *count, float *sum, int main_query, t_exp *experiment){
/* Reads the query results associated with query number qid 
   from file results. Then combines these results into count and sum. */

int qnumber, aux1, aux2, aux3, aux4;
long dnumber;
float similarity;

(*count)=0;
(*sum)=0;

/* First check for "leftovers" in gbuffer. If gbuffer is empty,
   then read the new line in it. */  

while((gbuffer[0]!='\0')||(fgets(gbuffer, MAXLINE, results)!=NULL)){
  sscanf(gbuffer, "%d %ld %d %d %d %d %f", 
         &qnumber, &dnumber, &aux1, &aux2, &aux3, &aux4, &similarity);

  if(qnumber==qid){
      /* results correspond to the right query */
      (*count)++;
      (*sum)=(*sum)+similarity;

      /* empty gbuffer: we've already used its contents */
      gbuffer[0]='\0';
  }
  else
    /* read first line for a later query: leave it in the gbuffer... */
    break;
}
}

/***********************************************************************/
float GlOSSGoodness(int num_terms, t_qres *query_results,
                    long num_docs, t_exp *experiment, int *query_weights){
/* Determines how "good" GlOSS would judge the current database for the
   current query to be, based on data for the individual keywords
   in the current query (given in query_results, with weights in 
   query_weights), and the algorithm in experiment. */

int i, j;
float goodness=0, added_weight=0;
long doc_count=0;
float weight_sum=0;

switch (experiment->gloss_rank_alg){
      
  case GLOSS_RANK_SUM_W:
  case GLOSS_RANK_SUM_D:
    /* GlOSS will assume that query words occur in documents with 
       negative correlation (Sum...) 
       GlOSS essentially assumes that no document contains 
       two query terms. */
    for(i=1; i<=num_terms; i++){
         if((query_results[i].doc_counts!=0)&&
            (query_weights[i-1]*
             (query_results[i].weight_sums/query_results[i].doc_counts) >
              experiment->threshold_gloss)){
           doc_count+=query_results[i].doc_counts;
           weight_sum+=query_results[i].weight_sums*query_weights[i-1];
         } 
    }

    if(experiment->gloss_rank_alg == GLOSS_RANK_SUM_W)
      goodness=weight_sum;
    else /* experiment->gloss_rank_alg == GLOSS_RANK_SUM_D */
      goodness=doc_count;

    break;  
      
  case GLOSS_RANK_MAX_W: 
  case GLOSS_RANK_MAX_D: 
    /* GlOSS will estimate the number of documents having weight > 
       threshold_gloss, using the high-correlation-of-keywords assumption. */ 

    qsort((void *)(query_results+1), (size_t)num_terms, 
         (size_t)(sizeof(t_qres)), (int (*)(void*,void*))query_results_compare);

    /* Now, find j (if it exists) such that:
       wj/fj + wj+1/fj+1 + ... + wn/fn  > threshold_gloss, but
               wj+1/fj+1 + ... + wn/fn <= threshold_gloss. 
     Then, under the assumptions, the only documents that will have
     weight > threshold_gloss are those containing (at least) terms 
     tj through tn. There are tj such documents. */

    added_weight=0;
    for(i=1; i<=num_terms; i++){
      if(query_results[i].doc_counts!=0)
        added_weight+=
            ((query_results[i].weight_sums)/(query_results[i].doc_counts));
    }
    
    if(added_weight<=(experiment->threshold_gloss)){
      /* no document (under the assumptions) has the necessary weight */
      doc_count=0;
      weight_sum=0;
    } 
    else{
      for(i=1; i<=num_terms; i++){
         if(query_results[i].doc_counts!=0)
           added_weight-=
               (query_results[i].weight_sums)/(query_results[i].doc_counts);
         if((added_weight<=(experiment->threshold_gloss))||(i==num_terms))
           break;
      }
      /* the weight sum of the documents that qualify is:
         w1+...+wj+ fj*(wj+1/fj+1 + ... + wn/fn) */
     
      doc_count=(query_results[i].doc_counts);  /* fj */
      weight_sum=doc_count*added_weight; /* fj*(wj+1/fj+1 + ... + wn/fn) */       
      for(j=1; j<=i; j++)
         weight_sum+=query_results[j].weight_sums;
    }
    
    if(experiment->gloss_rank_alg == GLOSS_RANK_MAX_W)
      goodness=weight_sum;
    else /* experiment->gloss_rank_alg == GLOSS_RANK_MAX_D */
      goodness=doc_count;

    break;

  default:
    Panic("Unknown GlOSS-rank algorithm");
    break;
} /* switch */

return goodness;
}

/***********************************************************************/
int query_results_compare(void *v1, void *v2)
{
 t_qres *t1, *t2;

 t1=(t_qres *)v1;
 t2=(t_qres *)v2;

 if((t1->doc_counts) < (t2->doc_counts))
   return -1;
 else{
   if((t1->doc_counts) == (t2->doc_counts))
     return 0;
   else
     return 1;
 }
}
 
/***********************************************************************/
void CompareRanks(int num_clusters, t_exp *experiment){
/* H_REAL_RANK contains, for each query, 
   the rank of database clusters according to their "goodness."
   HGLOSS_RANK contains, for each query, 
   the rank of database clusters according to HGlOSS predictions.
   experiment contains some parameters of the experiments.

   This function compares these two ranks query by query, 
   and prints statistics to the file specified in experiment. */

FILE *h_real, *hgloss;

OpenRankFiles(H_REAL_RANK, HGLOSS_RANK, &h_real, &hgloss);
ComputeStats(num_clusters, h_real, hgloss, experiment);
CloseRankFiles(h_real, hgloss);
}

/***********************************************************************/
void OpenRankFiles(char *f_real, char *f_gloss, 
                   FILE **real, FILE **gloss){

if((*real=fopen(f_real, "r"))==(FILE *)NULL)
  Panic("Couldn't open f_real file");

if((*gloss=fopen(f_gloss, "r"))==(FILE *)NULL)
  Panic("Couldn't open f_gloss file");
}

/***********************************************************************/
void CloseRankFiles(FILE *real, FILE *gloss){

fclose(real);
fclose(gloss);
}

/***********************************************************************/
void ComputeStats(int num_clusters, FILE *real, FILE *gloss, 
                  t_exp *experiment){

int num_queries=0;

float *goodness_real, *goodness_gloss, *predicted_goodness;

float goodness_recall[MAXCLUSTERS], goodness_precision[MAXCLUSTERS];  

CreateVectors(num_clusters, &goodness_real, &goodness_gloss, 
              &predicted_goodness, goodness_recall, goodness_precision);

while(ReadInOneQuery(num_clusters, real, gloss, 
             goodness_real, goodness_gloss, predicted_goodness)!=EOF){
   num_queries++;
   AnalyzeData(num_clusters, goodness_recall, goodness_precision, 
               goodness_real, goodness_gloss, predicted_goodness, experiment);
}

ReportResults(num_queries, num_clusters, goodness_recall, goodness_precision,
              experiment);
DestroyVectors(goodness_real, goodness_gloss, predicted_goodness);
}

/***********************************************************************/
void CreateVectors(int num_clusters, float **goodness_real, 
                   float **goodness_gloss, float **predicted_goodness,
                   float *goodness_recall, float *goodness_precision){
/* create vectors to keep the data on the two ranks for a single
   query */

int i;
int max_clusters;

*goodness_real     = (float *)malloc(sizeof(float)*num_clusters);
*goodness_gloss    = (float *)malloc(sizeof(float)*num_clusters);
*predicted_goodness= (float *)malloc(sizeof(float)*num_clusters);

/* initialize the statistics vectors */

if(num_clusters<MAXCLUSTERS)
  max_clusters=num_clusters;
else
  max_clusters=MAXCLUSTERS;

for(i=0; i<max_clusters; i++){
  goodness_recall[i]=0;
  goodness_precision[i]=0;
}
}

/***********************************************************************/
void DestroyVectors(float *goodness_real, float *goodness_gloss, 
                    float *predicted_goodness){

free(goodness_real);
free(goodness_gloss);
free(predicted_goodness);

}

/***********************************************************************/
int ReadInOneQuery(int num_clusters, FILE *real, FILE *gloss, 
                   float *goodness_real, float *goodness_gloss, 
                   float *predicted_goodness){

int qid, i;
char dbid[FILENAME_MAX];
long aux1;
float aux2, aux3;
char buffer[MAXLINE];

for(i=0; i<num_clusters; i++){

   if(fgets(buffer, MAXLINE, real)==NULL){
      if(i==0) 
        return EOF;
      else 
        Panic("Problem reading 'real' data");
   }

   if(sscanf(buffer, "%d %s %ld %f %f %f", &qid, dbid, 
                    &aux1, &aux2, &(goodness_real[i]), &aux3)!=6)
     Panic("Real rank in bad format");

   if((fgets(buffer, MAXLINE, gloss)==NULL)||
      (sscanf(buffer, "%d %s %ld %f %f %f", &qid, dbid, 
        &aux1, &aux2, &(goodness_gloss[i]), &(predicted_goodness[i]))!=6))
     Panic("Problem reading 'GlOSS' data");
}

return 0;
}

/***********************************************************************/
void AnalyzeData(int num_clusters, float *goodness_recall, 
                 float *goodness_precision,
                 float *goodness_real, float *goodness_gloss, 
                 float *predicted_goodness, t_exp *experiment){

/* Computes "recall" values at different cutting points: if we examine
   only the first n database clusters according to HGlOSS's rank, 
   what do we get compared with what we would have gotten with
   the "best" n clusters? Vector goodness_recall contains these recall
   values. Vector goodness_precision computes the fraction of useful
   clusters among the first n clusters in GlOSS's rank.  */

int n, i;
int max_clusters;
float ideal_goodness, gloss_goodness;
float new_recall, new_precision;
long chosen_clusters=0, useful_clusters=0; 
                                 /* to compute precision_n: in the
                                    top n clusters in the rank,
                                    there were chosen_clusters with
                                    non-zero estimate, and useful_clusters
                                    with non-zero goodness among them */

ideal_goodness=gloss_goodness=0;

if(num_clusters<MAXCLUSTERS)
  max_clusters=num_clusters;
else
  max_clusters=MAXCLUSTERS;

for(i=0; i<max_clusters; i++){
   ideal_goodness+=goodness_real[i];
   
   if(((experiment->prune)==NON_PRUNED_RANKS)||
      (predicted_goodness[i]>0)){
      /* will only add the real goodness of a database
         (to compute the overall goodness of the GlOSS rank)
         if that database was predicted to have non-zero goodness
         by GlOSS (PRUNED_RANKS option) */
      gloss_goodness+=goodness_gloss[i];
 
      chosen_clusters++;
      if(goodness_gloss[i]>0)
        useful_clusters++; 
   }
}

if(ideal_goodness==0)
  new_recall=1;
else{
  new_recall=gloss_goodness/ideal_goodness;
  if(new_recall>1) 
    Panic("Recall greater than one");
}

if(chosen_clusters==0)
  new_precision=1;
else{
  new_precision=useful_clusters/chosen_clusters;
  if(new_precision>1)
    Panic("Precision greater than one");
}

goodness_recall[max_clusters-1]+=new_recall;
goodness_precision[max_clusters-1]+=new_precision;

for(n=max_clusters-2; n>=0; n--){
  ideal_goodness-=goodness_real[n+1];

  if(((experiment->prune)==NON_PRUNED_RANKS)||
     (predicted_goodness[n+1]>0)){
    gloss_goodness-=goodness_gloss[n+1];

    chosen_clusters--;
    if(goodness_gloss[n+1]>0)
      useful_clusters--;
  }

  if(ideal_goodness==0)
    new_recall=1;
  else{
    new_recall=gloss_goodness/ideal_goodness;
    if(new_recall>1)
      Panic("Recall greater than one");
  }

  if(chosen_clusters==0)
    new_precision=1;
  else{
    new_precision=useful_clusters/chosen_clusters;
    if(new_precision>1)
      Panic("Precision greater than one");
  }
  goodness_recall[n]+=new_recall;
  goodness_precision[n]+=new_precision; 
}
}

/***********************************************************************/
void ReportResults(int num_queries, int num_clusters, 
                   float *goodness_recall, float *goodness_precision,
                   t_exp *experiment){

int i, max_clusters;
FILE *output;

if((output=fopen(experiment->output_file, "a"))==(FILE *)NULL)
   Panic("Can't open output file");

fprintf(output, "----------------------------------------------------------\n");
fprintf(output, "Queries:\t\t%d\n", num_queries);
fprintf(output, "Clusters:\t\t%d\n", num_clusters);
fprintf(output, "H_Real_rank_algorithm:\t%d\n", experiment->h_real_rank_alg);
fprintf(output, "GlOSS_rank_algorithm:\t%d\n", experiment->gloss_rank_alg);
fprintf(output, "HGlOSS_rank_algorithm:\t%d\n", experiment->hgloss_rank_alg);
fprintf(output, "Threshold_H_Real:\t%f\n", experiment->threshold_h_real);
fprintf(output, "Threshold_GlOSS:\t%f\n", experiment->threshold_gloss);
fprintf(output, "Threshold_HGlOSS:\t%f\n", experiment->threshold_hgloss);
if(experiment->prune == PRUNED_RANKS)
  /* No dbs with an estimated zero goodness are considered part of the rank */
  fprintf(output, "HGlOSS_rank pruned\n\n");
else
  fprintf(output, "HGlOSS_rank not pruned\n\n");

fprintf(output, "(1) Zero real goodness: \t %ld \n", zero_real_goodness);
fprintf(output, "(2) Zero HGlOSS goodness:\t %ld \n", zero_hgloss_goodness);
fprintf(output, "(3) Total goodness:\t\t %ld \n\n", total_goodness);
fprintf(output, "(1)/(3):\t %f \n", (zero_real_goodness*1.0)/total_goodness);
fprintf(output, "(2)/(3):\t %f \n\n", (zero_hgloss_goodness*1.0)/total_goodness);

fprintf(output, "Num_Clusters\tGoodness recall\tGoodness precision\n");

if(num_clusters<MAXCLUSTERS)
  max_clusters=num_clusters;
else
  max_clusters=MAXCLUSTERS;

for(i=0; i<max_clusters; i++)
   fprintf(output, "%d\t\t%f\t%f\n", i+1, goodness_recall[i]/num_queries,
                   goodness_precision[i]/num_queries); 

fprintf(output, "----------------------------------------------------------\n");
fclose(output);
}

/***********************************************************************/
void GenerateRanks(char *proc_cluster, char *f_queries, char *res_dir,
                   int *num_clusters, t_exp *experiment){
/* File proc_cluster has the definition of the different 
   database clusters: each cluster corresponds to one GlOSS server,
   and should be treated as a "unit" by HGlOSS.
   For each cluster, we generate a temporary CLUSTER_DATA file,
   having, for each query "a b c" and database db,
   a line for query "a b c" with the GlOSS estimated goodness
   of the query at db, plus a line for "a," a line for "b,"
   and a line for "c," each having the GlOSS information for db
   and each of the words. 
   We then collapse all these per-database entries into a single
   entry per query and cluster, in file H_RES_SUMMARY.

   Then generates the "real" rank, in file H_REAL_RANK, and
   the HGlOSS rank, in file HGLOSS_RANK, using the data in H_RES_SUMMARY.

   num_clusters returns the number of clusters in proc_cluster. */

char clusterid[FILENAME_MAX];
char dbid[FILENAME_MAX];
char buffer[MAXLINE];
FILE *tmp;
int num_dbs;

if((tmp=fopen(proc_cluster, "r"))==(FILE *)NULL)
  Panic("Couldn't open proc_cluster file");

/* clean up summary file */
remove(H_RES_SUMMARY);

(*num_clusters)=0;

/* process the data from each cluster separately */
while((fgets(buffer, FILENAME_MAX, tmp)!=NULL)&&
      (sscanf(buffer, "%s", clusterid)==1)){
   printf("DEBUG: processing cluster %s\n", clusterid);

   (*num_clusters)++;

   /* process cluster clusterid */
   remove(CLUSTER_DATA);

   /* initialize database count for the current cluster */ 
   num_dbs=0;

   /* process the data from each database separately */
   /* the end of the db cluster is indicated by "-------" */ 
   while((fgets(buffer, MAXLINE, tmp)!=NULL)&& 
         (sscanf(buffer, "%s", dbid)==1)&&
         (strstr(dbid, "----")==NULL)){
     /* Now process the data for this database... */
     printf("\tDEBUG: processing database %s\n", dbid);

     ProcessQueryResults(dbid, 0, f_queries, res_dir, experiment);
     num_dbs++;
   }

   /* Now collapse the data in CLUSTER_DATA into a single summary line
      per query and write out to H_RES_SUMMARY */
   CollapseClusterData(clusterid, num_dbs, 
                       CLUSTER_DATA, H_RES_SUMMARY, experiment);
}

sprintf(buffer, "%s \"%s\" \"%s\" \"%s\"\n", RANKER, 
        H_RES_SUMMARY, H_REAL_RANK, HGLOSS_RANK);
system(buffer);
}

/***********************************************************************/
void CollapseClusterData(char *clusterid, int num_dbs, char *cluster_data,
                         char *h_res_summary, t_exp *experiment){

/* Collapses all the entries in CLUSTER_DATA into one summary line
   per query into H_RES_SUMMARY, using the algorithms described in
   experiment. */ 

char buffer[MAXLINE];
char f_cluster_data[FILENAME_MAX];
FILE *data, *summary;
int qid, num_terms, qid2, num_terms2;
char dbid[FILENAME_MAX];
float estimate, sum_est, sum_weight_term, weight;
long db_count, db_count_term, count;
int i, j;

long db_estimate=0;
float weight_estimate=0;

float real_goodness, hgloss_goodness;

sprintf(f_cluster_data, "%s.srt", cluster_data);

/* Sort the data on the different databases and queries by query id,
   the first field of each line */
sprintf(buffer, "sort -n %s > %s\n", cluster_data, f_cluster_data);
system(buffer);

if((data=fopen(f_cluster_data, "r"))==(FILE *)NULL)
  Panic("Can't open sorted cluster data file");

if((summary=fopen(h_res_summary, "a"))==(FILE *)NULL)
  Panic("Can't append to h_res_summary file");

/* Scan f_cluster_data and collapse the information about each query */

while(fgets(buffer, MAXLINE, data)!=NULL){
  /* process query */
  if(sscanf(buffer, "%d %d %s %f", &qid, &num_terms, dbid, &estimate)!=4)
    Panic("Wrong format of CLUSTER_DATA line");

  if(estimate>experiment->threshold_h_real){
    sum_est=estimate;
    db_count=1;
  }
  else{
    sum_est=0;
    db_count=0;
  }

  for(i=1; i<num_dbs; i++){
     /* read results from the remaining num_dbs-1 databases and 
        accumulate, to compute h_real_goodness of the cluster later */
     if((fgets(buffer, MAXLINE, data)==NULL)||
        (sscanf(buffer, "%d %d %s %f", &qid2, &num_terms2, dbid, 
         &estimate)!=4)||(qid!=qid2)||(num_terms!=num_terms2))
       Panic("Problem with CLUSTER_DATA line");

     if(estimate>experiment->threshold_h_real){
       sum_est+=estimate;
       db_count++;
     } 
  }

  /* Compute HGlOSS estimates */

  /* Reset value of HGlOSS estimates */
  weight_estimate=0;
  db_estimate=0;

  for(i=0; i<num_terms; i++){
     /* read results for the individual query terms and use
        them for the HGlOSS estimate */
    
     sum_weight_term=0;
     db_count_term=0;

     for(j=0; j<num_dbs; j++){
        /* read the data from each database and collapse */
        if((fgets(buffer, MAXLINE, data)==NULL)||
           (sscanf(buffer, "%d %s %f %ld", &qid2, dbid, &weight, &count)!=4))
           Panic("Problem with CLUSTER_DATA line");

        if(count>0){
          sum_weight_term+=(weight/count);
          db_count_term++;
        }
     }

     weight_estimate+=sum_weight_term;
     if(db_count_term>db_estimate)
       db_estimate=db_count_term;
  }

  if(experiment->h_real_rank_alg==H_REAL_RANK_W)
    real_goodness=sum_est;
  else
    real_goodness=db_count; 

  if(experiment->hgloss_rank_alg==H_GLOSS_RANK_MAX_W)
    hgloss_goodness=weight_estimate;
  else
    hgloss_goodness=db_estimate; 

  /* Output one-line summary for the cluster and the current query */
  fprintf(summary, "%d\t%s %ld %f %f %f\n", 
          qid, clusterid, db_count, sum_est, real_goodness, hgloss_goodness);

  if(real_goodness==0)
    zero_real_goodness++;

  if(hgloss_goodness==0)
    zero_hgloss_goodness++;

  total_goodness++;

}

fclose(data);
fclose(summary);
}

/***********************************************************************/
