
void PreprocessSiftQueries(char *filename);
/* file filename.all contains lines like:

   Profile hello
   Type    0
   Profile goodbye
   Type    1

  These are SIFT profiles (see http://sift.stanfor.edu).
  Type 0 indicates vector-space profiles.
  Type 1 indicates boolean profiles.
  So far, we're just interested in the vector-space profiles.
  These are moved to file filename.prs */

void GenerateSmartQueries(char *filename);
/* filename.prs contains a parsed set of queries of the form:

   w_1  w_2  ... w_n1
   w_1' w_2' ... w_n2'
   ...

   This function generates file filename.smr, containing
   a set of queries for Smart that are parseable with
   the generic Smart parser, and including all single-word 
   queries after each of the "big" queries. It also generates
   file filename.mst with all of the queries without the Smart
   separators, for later use... */

void ProcessDBList(char *dblist);

/* Takes each directory dir (relative to the NEWS_DIR directory)
   listed in file dblist and 
   generates an entry (dir, DB_ID(dir)) in a new file dblist.proc.
   DB_ID(dir) is the database id that will be associated with
   the database that dir specifies */

void ProcessQueryResults(char *dbid, long num_docs, 
            char *f_queries, char *res_dir, t_exp *experiment);
/* f_queries.mst is a file containing the original set of queries.
   res_dir is a directory containing the result of evaluating the 
   queries in f_queries.mst on database dbid, which has
   num_docs in total.
   File res_dir/RES_SUMMARY will contain the "summary" information
   (see below) for each query and database dbid.

   This function processes the query results and comes up with two 
   "numbers" for each query: the first number captures the "goodness"
   of dbid for the query,
   and the second number is GlOSS's attempt at capturing this goodness.
   (These algorithms are specified in experiment.) */

void CompareRanks(int num_dbs, t_exp *experiment);
/* REAL_RANK contains, for each query, 
   the rank of databases according to their "goodness."
   GLOSS_RANK contains, for each query, 
   the rank of databases according to GlOSS predictions.
   experiment contains some parameters of the experiments.

   This function compares these two ranks query by query, 
   and prints statistics to the file specified in experiment. */

void FeedSmart(char *proc_dblist, int from, int to, char *res_dir);
/* Takes each <db, dbid> pair in proc_dblist, and feeds it to the
   test_news... scripts, which generate the corresponding Smart database.
   Actually, process only pairs from (and including) 'from' to
   (and including) 'to.' Number dbs starting from 0.

   After indexing each database, process the results from the binaries
   that Smart generated in directory res_dir, and generate the corresponding
   .txt files containing ASCII versions of the query results. */

void GenerateRanks(char *proc_dblist, char *f_queries, char *res_dir,
                   int *num_dbs, t_exp *experiment);
/* First generates a single RES_SUMMARY file in res_dir, with
   processed query results according to the ranking algorithms
   being used (specified in experiment).

   Then generates the "real" rank, in file res_dir/REAL_RANK, and
   the GlOSS rank, in file res_dir/GLOSS_RANK, using the
   data in res_dir/RES_SUMMARY

   num_dbs returns the number of databases in proc_dblist. */ 

